from logging.config import fileConfig
from os import path

import yaml
from sqlalchemy import engine_from_config
from sqlalchemy import pool

from alembic import context

from core import CoreConfig

# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config

# Interpret the config file for Python logging.
# This line sets up loggers basically.
if config.config_file_name is not None:
    fileConfig(config.config_file_name, disable_existing_loggers=False)

# add your model's MetaData object here
# for 'autogenerate' support
# from myapp import mymodel
# target_metadata = mymodel.Base.metadata

# Core
from core.data.schema import metadata

# Titles
from titles.chuni.schema import *
from titles.cm.schema import *
from titles.cxb.schema.item import *
from titles.diva.schema import *
from titles.idac.schema import *
# from titles.idz.schema import *
from titles.mai2.schema import *
from titles.ongeki.schema import *
from titles.pokken.schema import *
from titles.sao.schema import *

target_metadata = metadata

# other values from the config, defined by the needs of env.py,
# can be acquired:
# my_important_option = config.get_main_option("my_important_option")
# ... etc.


def run_migrations_offline() -> None:
    """Run migrations in 'offline' mode.

    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.

    Calls to context.execute() here emit the given string to the
    script output.

    """
    url = config.get_main_option("sqlalchemy.url")

    context.configure(
        url=url,
        target_metadata=target_metadata,
        literal_binds=True,
        dialect_opts={"paramstyle": "named"},
    )

    with context.begin_transaction():
        context.run_migrations()


def get_url_from_config():
    url = config.get_main_option('artemis_db_url')
    if url is not None:
        return url

    import core.data.database

    cfg: CoreConfig = CoreConfig()
    if path.exists(f"config/core.yaml"):
        cfg.update(yaml.safe_load(open(f"config/core.yaml")))
    return core.data.database.build_db_url_from_config(cfg.database)


def run_migrations_online() -> None:
    """Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.

    """
    # Load config
    url = get_url_from_config()

    from sqlalchemy import create_engine
    engine = create_engine(url)
    with engine.connect() as connection:
        context.configure(
            connection=connection, target_metadata=target_metadata
        )

        with context.begin_transaction():
            context.run_migrations()


if context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()
